import "./styles.sass";
import {SocketContext} from "@/common/contexts/SocketContext";
import {useContext, useState, useEffect, useRef} from "react";
import MusicSlider from "@/pages/Game/components/MusicSlider";
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import {faMessage, faMusic, faHeadphones, faClock, faCrown} from "@fortawesome/free-solid-svg-icons";

export const Game = () => {
    const {send, on, socket} = useContext(SocketContext);

    const [role, setRole] = useState(null); // 'composer' or 'guesser'
    const [round, setRound] = useState(1);
    const [phase, setPhase] = useState("waiting"); // waiting, composing, guessing, results
    const [timeLeft, setTimeLeft] = useState(30);
    const [frequency, setFrequency] = useState(440);
    const [currentSong, setCurrentSong] = useState(null);
    const [songOptions, setSongOptions] = useState([]);
    const [scores, setScores] = useState({});
    const [selectedSong, setSelectedSong] = useState(null);
    const [guessResult, setGuessResult] = useState(null);
    const [isHost, setIsHost] = useState(false);
    
    // Chat state
    const [messages, setMessages] = useState([]);
    const [inputValue, setInputValue] = useState("");
    const [connectedUsers, setConnectedUsers] = useState([]);
    const [username, setUsername] = useState("");
    const messageEndRef = useRef(null);
    const timerIntervalRef = useRef(null);

    useEffect(() => {
        setPhase("waiting");
        
        const handleRolesAssigned = (roles) => {
            console.log("Roles assigned:", roles);
            const myRole = roles[socket?.id];
            
            if (myRole) {
                setRole(myRole);

                setMessages(prev => [...prev, {
                    system: true,
                    text: myRole === "composer" 
                        ? "Du bist der Komponist! Spiele den Song mit dem Tonregler." 
                        : "Du bist ein Rater! Höre die Frequenzen und versuche, den Song zu erkennen."
                }]);
            } else {
                console.error("No role assigned to this player!");
            }
        };
        
        const handleSongSelected = (song) => {
            console.log("Song selected:", song);
            setCurrentSong(song);
        };
        
        const handleRoundStarted = (data) => {
            console.log("Round started:", data);
            setRound(data.round);
            setPhase("composing");
            setTimeLeft(data.timeRemaining);
        };
        
        const handleGuessingPhaseStarted = (data) => {
            console.log("Guessing phase started:", data);
            setPhase("guessing");
            setTimeLeft(data.timeRemaining);
            setSongOptions(data.songOptions);
        };
        
        const handleGuessResult = (result) => {
            console.log("Guess result:", result);
            setGuessResult(result.isCorrect);
            setCurrentSong(result.correctSong);
        };
        
        const handleRoundResults = (results) => {
            console.log("Round results:", results);
            setPhase("results");
            setScores(results.scores);
            if (!currentSong) {
                setCurrentSong(results.selectedSong);
            }
        };
        
        const handleRoomUsers = (users) => {
            console.log("Room users:", users);
            setConnectedUsers(users);

            const currentUser = users.find(u => u.id === socket?.id);
            if (currentUser) {
                setUsername(currentUser.name);
            }
        };
        
        const handleHostStatus = (status) => {
            setIsHost(status.isHost);
        };

        const cleanupRolesAssigned = on("roles-assigned", handleRolesAssigned);
        const cleanupSongSelected = on("song-selected", handleSongSelected);
        const cleanupRoundStarted = on("round-started", handleRoundStarted);
        const cleanupGuessingPhaseStarted = on("guessing-phase-started", handleGuessingPhaseStarted);
        const cleanupGuessResult = on("guess-result", handleGuessResult);
        const cleanupRoundResults = on("round-results", handleRoundResults);
        const cleanupRoomUsers = on("room-users-update", handleRoomUsers);
        const cleanupHostStatus = on("host-status", handleHostStatus);

        send("get-room-users");

        send("check-host-status");
        
        return () => {
            cleanupRolesAssigned();
            cleanupSongSelected();
            cleanupRoundStarted();
            cleanupGuessingPhaseStarted();
            cleanupGuessResult();
            cleanupRoundResults();
            cleanupRoomUsers();
            cleanupHostStatus();
        };
    }, [socket, on, send]);

    useEffect(() => {
        if (timerIntervalRef.current) {
            clearInterval(timerIntervalRef.current);
        }

        if (phase !== "waiting" && phase !== "results") {
            timerIntervalRef.current = setInterval(() => {
                setTimeLeft(prev => Math.max(0, prev - 1));
            }, 1000);
        }
        
        return () => {
            if (timerIntervalRef.current) {
                clearInterval(timerIntervalRef.current);
            }
        };
    }, [phase]);

    useEffect(() => {
        const handleChatMessage = (messageData) => {
            setMessages(prev => [...prev, messageData]);
        };
        
        const handleUserConnected = (userData) => {
            setMessages(prev => [...prev, {
                system: true,
                text: `${userData.name} ist beigetreten`
            }]);
            setConnectedUsers(prev => [...prev, userData]);
        };
        
        const handleUserDisconnected = (userId) => {
            setConnectedUsers(prev => {
                const user = prev.find(u => u.id === userId);
                if (user) {
                    setMessages(prevMsgs => [...prevMsgs, {
                        system: true,
                        text: `${user.name} hat den Raum verlassen`
                    }]);
                }
                return prev.filter(u => u.id !== userId);
            });
        };

        const handleFrequencyUpdate = (data) => {
            if (role === "guesser") {
                setFrequency(data.frequency);
            }
        };

        const cleanupChatMessage = on("chat-message", handleChatMessage);
        const cleanupUserConnected = on("user-connected", handleUserConnected);
        const cleanupUserDisconnected = on("user-disconnected", handleUserDisconnected);
        const cleanupFrequencyUpdate = on("frequency-update", handleFrequencyUpdate);
        
        return () => {
            cleanupChatMessage();
            cleanupUserConnected();
            cleanupUserDisconnected();
            cleanupFrequencyUpdate();
        };
    }, [on, role]);

    useEffect(() => {
        messageEndRef.current?.scrollIntoView({behavior: "smooth"});
    }, [messages]);

    const handleFrequencyChange = (newFrequency) => {
        setFrequency(newFrequency);
        if (role === "composer") {
            send("submit-frequency", { frequency: newFrequency });
        }
    };
    
    const handleSendMessage = () => {
        if (inputValue.trim()) {
            const messageData = {
                text: inputValue,
                sender: username
            };
            send("send-message", messageData);
            setMessages(prev => [...prev, messageData]);
            setInputValue("");
        }
    };
    
    const handleSongSelect = (song) => {
        setSelectedSong(song);
    };
    
    const handleNextRound = () => {
        send("next-round");

        setSelectedSong(null);
        setGuessResult(null);
        setTimeLeft(0);
    };

    const renderPhaseContent = () => {
        switch (phase) {
            case "waiting":
                return (
                    <div className="waiting-phase">
                        <h3>Warten auf Spielstart...</h3>
                    </div>
                );
                
            case "composing":
                return (
                    <div className="composing-phase">
                        <div className="phase-header">
                            <h3>Runde {round}: {role === "composer" ? "Spielen" : "Zuhören"}</h3>
                            <div className="timer">
                                <FontAwesomeIcon icon={faClock} /> {timeLeft}s
                            </div>
                        </div>
                        
                        {role === "composer" && currentSong && (
                            <div className="song-display">
                                <div className="song-card">
                                    <img src={currentSong.coverUrl} alt={currentSong.title} />
                                    <div className="song-info">
                                        <div className="song-names">{currentSong.title}</div>
                                        <div className="song-description">von {currentSong.artist}</div>
                                    </div>
                                </div>
                                <p className="instruction">Spiele diesen Song mit dem Tonregler!</p>
                            </div>
                        )}
                        
                        {role === "guesser" && (
                            <div className="listening-display">
                                <div className="listening-icon">
                                    <FontAwesomeIcon icon={faHeadphones} size="4x" />
                                </div>
                                <p className="instruction">Höre genau zu und versuche, den Song zu erkennen!</p>
                            </div>
                        )}
                    </div>
                );
                
            case "guessing":
                return (
                    <div className="guessing-phase">
                        <div className="phase-header">
                            <h3>Runde {round}: Song erraten</h3>
                            <div className="timer">
                                <FontAwesomeIcon icon={faClock} /> {timeLeft}s
                            </div>
                        </div>
                        
                        {role === "composer" && (
                            <div className="waiting-for-guessers">
                                <p>Die Rater versuchen nun, deinen Song zu erraten...</p>
                            </div>
                        )}
                        
                        {role === "guesser" && (
                            <div className="song-options">
                                <p className="instruction">Welchen Song hat der Komponist gespielt?</p>
                                <div className="song-grid">
                                    {songOptions.map(song => (
                                        <div 
                                            key={song.id}
                                            className={`song-option ${selectedSong?.id === song.id ? 'selected' : ''}`}
                                            onClick={() => handleSongSelect(song)}
                                        >
                                            <img src={song.coverUrl} alt={song.title} />
                                            <div className="song-details">
                                                <div className="song-title">{song.title}</div>
                                                <div className="song-artist">{song.artist}</div>
                                            </div>
                                        </div>
                                    ))}
                                </div>
                            </div>
                        )}
                    </div>
                );
                
            case "results":
                return (
                    <div className="results-phase">
                        <h3>Runde {round}: Ergebnisse</h3>
                        
                        <div className="round-results">
                            {role === "composer" && (
                                <div className="composer-results">
                                    <p>Die Rater haben versucht, deinen Song zu erraten.</p>
                                </div>
                            )}
                            
                            {role === "guesser" && (
                                <div className="guesser-results">
                                    {currentSong && (
                                        <div className="correct-song">
                                            <p>Der richtige Song war:</p>
                                            <div className="song-card highlight">
                                                <img src={currentSong.coverUrl} alt={currentSong.title} />
                                                <div className="song-info">
                                                    <div className="song-names">{currentSong.title}</div>
                                                    <div className="song-description">von {currentSong.artist}</div>
                                                </div>
                                            </div>
                                        </div>
                                    )}
                                    
                                    {guessResult !== null && (
                                        <div className={`guess-result ${guessResult ? 'correct' : 'incorrect'}`}>
                                            {guessResult 
                                                ? 'Richtig! Du erhälst 10 Punkte.' 
                                                : 'Leider falsch. Kein Punkt.'}
                                        </div>
                                    )}
                                </div>
                            )}
                            
                            <div className="scoreboard">
                                <h4>Punktestand</h4>
                                <div className="scores">
                                    {Object.entries(scores).map(([userId, score]) => {
                                        const user = connectedUsers.find(u => u.id === userId) || { name: 'Unbekannt' };
                                        return (
                                            <div key={userId} className="score-entry">
                                                <span className="player-name">
                                                    {user.id === (socket?.id || "you") && "👉 "}
                                                    {user.name}
                                                    {userId === connectedUsers.find(u => u.id === "user1")?.id && (
                                                        <FontAwesomeIcon icon={faCrown} className="host-icon" />
                                                    )}
                                                </span>
                                                <span className="player-score">{score}</span>
                                            </div>
                                        );
                                    })}
                                </div>
                            </div>
                            
                            {isHost && (
                                <button className="next-round-button" onClick={handleNextRound}>
                                    Nächste Runde
                                </button>
                            )}
                            {!isHost && <p className="waiting-message">Warten auf Rundenwechsel durch Host...</p>}
                        </div>
                    </div>
                );
                
            default:
                return <div>Unknown phase</div>;
        }
    };

    return (
        <div className="game-page">
            <div className="background-overlay">
                <div className="rotating-gradient"></div>
            </div>
            
            <div className="game-header">
                <div className="game-role">
                    <FontAwesomeIcon icon={role === "composer" ? faMusic : faHeadphones} />
                    <span>{role === "composer" ? "Komponist" : "Rater"}</span>
                </div>
                <h2>ToneGuessr</h2>
                <div className="game-round">Runde {round}</div>
            </div>
            
            <div className="main-content">
                {renderPhaseContent()}
                
                <div className="chat-window">
                    <div className="chat-header">
                        <FontAwesomeIcon icon={faMessage} />
                        <div className="chat-title">Chat</div>
                    </div>
                    <div className="chat-messages">
                        {messages.map((message, index) => (
                            <div key={index} className={`message ${message.system ? 'system-message' : ''}`}>
                                {message.system ? (
                                    <span className="message-text system">{message.text}</span>
                                ) : (
                                    <>
                                        <span className="message-sender">{message.sender}:</span>
                                        <span className="message-text">{message.text}</span>
                                    </>
                                )}
                            </div>
                        ))}
                        <div ref={messageEndRef}></div>
                    </div>
                    <div className="chat-input">
                        <input type="text" value={inputValue} 
                               onChange={(e) => setInputValue(e.target.value)}
                               onKeyPress={(e) => e.key === 'Enter' && handleSendMessage()}
                               placeholder="Gib eine Nachricht ein..."
                        />
                        <button onClick={handleSendMessage}>Send</button>
                    </div>
                </div>
            </div>
            
            <MusicSlider 
                isReadOnly={role !== "composer" || phase !== "composing"} 
                onFrequencyChange={handleFrequencyChange}
                frequency={frequency}
            />
        </div>
    );
}